package commands;



public class MulCommand extends AICommand {
	
	@Override
	public String getShortName() {
		return "MUL";
	}
	
	@Override
	public String getLongName() {
		return "*";
	}
	
	@Override
	public int getParameterCount() {
		return 2;
	}
	
	@Override
	public int doCommand() {
		return doCommand(true);
	}
	
	@Override
	public int doCommand(boolean withSideEffects) {
		if (stop) {
			return 0;
		}
		int ret = 0;
		logExecute(this.getClass().getName() + ".doCommand()", getLongName() + "(");
		if (getParameter(0) != null) {
			ret = getParameter(0).doCommand(withSideEffects);
		} else {
			logError(this.getClass().getName() + ".doCommand()", "Missing first parameter");
		}
		if (stop) {
			return 0;
		}
		logExecute(this.getClass().getName() + ".doCommand()", ",");
		if (getParameter(1) != null) {
			double a = (double) ret;
			double b = (double) getParameter(1).doCommand(withSideEffects);
			double r = (a/1000) * (b/1000);
			ret = (int) (r*1000);
		} else {
			logError(this.getClass().getName() + ".doCommand()", "Missing second parameter");
		}
		logExecute(this.getClass().getName() + ".doCommand()", ")" + getLongName() + " = " + ret);
		return ret;
	}

	public MulCommand() {
		super();
	}

	public MulCommand(MulCommand src) {
		super(src);
	}

	@Override
	public IAICommand isThisCommand(String string) {
		if (string.equals(getShortName())) {
			return new MulCommand(this);
		}
		return null;
	}

	@Override
	public IAICommand copy() {
		return new MulCommand(this);
	}

	@Override
	public IAICommand copyRandom() {
		return copy();
	}

	@Override
	public boolean isConstant() {
		// all parameters must be const to be const
		boolean ret = true;
		for (int i=0; i<this.getCurrentParameterCount(); i++) {
			ret = ret && ((AICommand)this.getParameter(i)).isConstant();
		}
		return ret;
	}
	
	@Override
	public IsExecutedType isParameterExecuted(int paramID) {
		return IsExecutedType.ALWAYS;
	}

	@Override
	public boolean hasSideEffects(boolean inclusiveParameter) {
		boolean ret = false;
		if (inclusiveParameter) {
			for (int i=0; i<this.getCurrentParameterCount(); i++) {
				ret = ret || ((AICommand)this.getParameter(i)).hasSideEffects(inclusiveParameter);
			}
		}
		return ret;
	}
}
